@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU Library General Public License
  * along with this library; if not, write to the Free Software Foundation,
- * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. *
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #include <stdio.h>
@@ -31,14 +31,13 @@
  *
  * Summary:     Unpack the AppInfo block into the structure
  *
- * Parameters:  None
+ * Parameters:  CategoryAppInfo_t*, char* to record, record length
  *
  * Returns:     Nothing
  *
  ***********************************************************************/
 int
-unpack_CategoryAppInfo(struct CategoryAppInfo *ai, unsigned char *record,
-		       int len)
+unpack_CategoryAppInfo(CategoryAppInfo_t *ai, unsigned char *record, size_t len)
 {
 	int 	i,
 		rec;
@@ -70,14 +69,13 @@ unpack_CategoryAppInfo(struct CategoryAppInfo *ai, unsigned char *record,
  *
  * Summary:     Pack the AppInfo structure 
  *
- * Parameters:  None
+ * Parameters:  CategoryAppInfo_t*, char* to record, record length
  *
  * Returns:     Nothing
  *
  ***********************************************************************/
 int
-pack_CategoryAppInfo(struct CategoryAppInfo *ai, unsigned char *record,
-		     int len)
+pack_CategoryAppInfo(CategoryAppInfo_t *ai, unsigned char *record, size_t len)
 {
 	int 	i,
 		rec;
