@@ -15,18 +15,18 @@
  *
  * You should have received a copy of the GNU Library General Public License
  * along with this library; if not, write to the Free Software Foundation,
- * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. *
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
-#if TIME_WITH_SYS_TIME 	 
+#ifdef TIME_WITH_SYS_TIME 	 
 # include <sys/time.h> 	 
 # include <time.h> 	 
 #else 	 
-# if HAVE_SYS_TIME_H 	 
+# ifdef HAVE_SYS_TIME_H 	 
 #  include <sys/time.h> 	 
 # else 	 
 #  include <time.h> 	 
@@ -36,6 +36,12 @@
 #include "pi-macros.h"
 #include "pi-datebook.h"
 
+#define alarmFlag 	64
+#define repeatFlag 	32
+#define noteFlag 	16
+#define exceptFlag 	8
+#define descFlag 	4
+
 char *DatebookAlarmTypeNames[] = { "Minutes", "Hours", "Days", NULL };
 
 char *DatebookRepeatTypeNames[] =
@@ -52,21 +58,30 @@ char *DatebookRepeatTypeNames[] =
  *
  * Function:    free_Appointment
  *
- * Summary:     Frees members of the apointment structure
+ * Summary:     Frees members of the appointment structure
  *
- * Parameters:  Appointment structure
+ * Parameters:  Appointment_t*
  *
- * Returns:     Nothing
+ * Returns:     void
  *
  ***********************************************************************/
-void free_Appointment(struct Appointment *a)
+void
+free_Appointment(Appointment_t *a)
 {
-	if (a->exception)
+	if (a->exception != NULL) {
 		free(a->exception);
-	if (a->description)
+		a->exception = NULL;
+	}
+
+	if (a->description != NULL) {
 		free(a->description);
-	if (a->note)
+		a->description = NULL;
+	}
+
+	if (a->note != NULL) {
 		free(a->note);
+		a->note = NULL;
+	}
 }
 
 /***********************************************************************
@@ -76,14 +91,13 @@ void free_Appointment(struct Appointment *a)
  * Summary:     Fill in the appointment structure based on the raw 
  *		record data
  *
- * Parameters:  None
+ * Parameters:  Appointment_t*, pi_buffer_t * of buffer, datebook type
  *
- * Returns:     0 on error, the length of the data used from the
- *		buffer otherwise
+ * Returns:     -1 on fail, 0 on success
  *
  ***********************************************************************/
 int
-unpack_Appointment(struct Appointment *a, unsigned char *buffer, int len)
+unpack_Appointment(Appointment_t *a, pi_buffer_t *buf, datebookType type)
 {
 	int 	iflags,
 		j,
@@ -108,23 +122,27 @@ unpack_Appointment(struct Appointment *a, unsigned char *buffer, int len)
 	   -- KJA */
 
 	destlen = 8;
-	if (len < destlen)
-		return 0;
 
-	a->begin.tm_hour 	= get_byte(buffer);
-	a->begin.tm_min 	= get_byte(buffer + 1);
+	if (type != datebook_v1)
+		return -1;
+
+	if (buf == NULL || buf->data == NULL || buf->used < destlen)
+		return -1;
+
+	a->begin.tm_hour 	= get_byte(buf->data);
+	a->begin.tm_min 	= get_byte(buf->data + 1);
 	a->begin.tm_sec 	= 0;
-	d = (unsigned short int) get_short(buffer + 4);
+	d = (unsigned short int) get_short(buf->data + 4);
 	a->begin.tm_year 	= (d >> 9) + 4;
 	a->begin.tm_mon 	= ((d >> 5) & 15) - 1;
 	a->begin.tm_mday 	= d & 31;
 	a->begin.tm_isdst 	= -1;
 	a->end = a->begin;
 
-	a->end.tm_hour = get_byte(buffer + 2);
-	a->end.tm_min = get_byte(buffer + 3);
+	a->end.tm_hour = get_byte(buf->data + 2);
+	a->end.tm_min = get_byte(buf->data + 3);
 
-	if (get_short(buffer) == 0xffff) {
+	if (get_short(buf->data) == 0xffff) {
 		a->event 		= 1;
 		a->begin.tm_hour 	= 0;
 		a->begin.tm_min 	= 0;
@@ -137,17 +155,11 @@ unpack_Appointment(struct Appointment *a, unsigned char *buffer, int len)
 	mktime(&a->begin);
 	mktime(&a->end);
 
-	iflags = get_byte(buffer + 6);
+	iflags = get_byte(buf->data + 6);
 
-	/* buffer+7 is gapfill */
+	/* buf->data+7 is gapfill */
 
-	p2 = (unsigned char *) buffer + 8;
-
-#define alarmFlag 	64
-#define repeatFlag 	32
-#define noteFlag 	16
-#define exceptFlag 	8
-#define descFlag 	4
+	p2 = (unsigned char *) buf->data + 8;
 
 	if (iflags & alarmFlag) {
 		a->alarm 	= 1;
@@ -234,18 +246,18 @@ unpack_Appointment(struct Appointment *a, unsigned char *buffer, int len)
 	}
 
 	if (iflags & descFlag) {
-		a->description = strdup(p2);
+		a->description = strdup((char *)p2);
 		p2 += strlen((char *)p2) + 1;
 	} else
 		a->description = 0;
 
 	if (iflags & noteFlag) {
-		a->note = strdup(p2);
+		a->note = strdup((char *)p2);
 		p2 += strlen((char *)p2) + 1;
 	} else {
 		a->note = 0;
 	}
-	return (p2 - buffer);
+	return 0;
 }
 
 /***********************************************************************
@@ -253,21 +265,29 @@ unpack_Appointment(struct Appointment *a, unsigned char *buffer, int len)
  * Function:    pack_Appointment
  *
  * Summary:	Fill in the raw appointment record data based on the 
- *		appointment structure
+ *		appointment structure.
  *
- * Parameters:  None
+ * Parameters:  Appointment_t*, pi_buffer_t*, datebook database version
+ *              from the datebookType enumeration -- only v1 is supported
+ *              for now.
  *
- * Returns:     The length of the buffer required if record is NULL,
- *		or 0 on error, the length of the data used from the 
- *		buffer otherwise
+ * Returns:     -1 on error (bad arguments, mostyle) or 0 on success.
+ *              The buffer is sized to accomodate the required data.
  *
  ***********************************************************************/
-int pack_Appointment(struct Appointment *a, unsigned char *buf, int len)
+int
+pack_Appointment(Appointment_t *a, pi_buffer_t *buf, datebookType type)
 {
 	int 	iflags,
 		destlen = 8;
 	char 	*pos;
 
+	if (a == NULL || buf == NULL)
+		return -1;
+
+	if (type != datebook_v1)
+		return -1;
+
 	if (a->alarm)
 		destlen += 2;
 	if (a->repeatType)
@@ -279,33 +299,26 @@ int pack_Appointment(struct Appointment *a, unsigned char *buf, int len)
 	if (a->description)
 		destlen += strlen(a->description) + 1;
 
-	if (!buf)
-		return destlen;
-	if (len < destlen)
-		return 0;
+	pi_buffer_expect (buf, destlen);
+	buf->used = destlen;	
 
-	set_byte(buf, a->begin.tm_hour);
-	set_byte(buf + 1, a->begin.tm_min);
-	set_byte(buf + 2, a->end.tm_hour);
-	set_byte(buf + 3, a->end.tm_min);
-	set_short(buf + 4,
+	set_byte(buf->data, a->begin.tm_hour);
+	set_byte(buf->data + 1, a->begin.tm_min);
+	set_byte(buf->data + 2, a->end.tm_hour);
+	set_byte(buf->data + 3, a->end.tm_min);
+	set_short(buf->data + 4,
 		  ((a->
 		    begin.tm_year - 4) << 9) | ((a->begin.tm_mon +
 						 1) << 5) | a->begin.
 		  tm_mday);
 
 	if (a->event) {
-		set_long(buf, 0xffffffff);
+		set_long(buf->data, 0xffffffff);
 	}
-#define alarmFlag 	64
-#define repeatFlag 	32
-#define noteFlag 	16
-#define exceptFlag 	8
-#define descFlag 	4
 
 	iflags = 0;
 
-	pos = (char *) buf + 8;
+	pos = (char *) buf->data + 8;
 
 	if (a->alarm) {
 		iflags |= alarmFlag;
@@ -388,10 +401,10 @@ int pack_Appointment(struct Appointment *a, unsigned char *buf, int len)
 		pos += strlen(pos) + 1;
 	}
 
-	set_byte(buf + 6, iflags);
-	set_byte(buf + 7, 0);	/* gapfill */
+	set_byte(buf->data + 6, iflags);
+	set_byte(buf->data + 7, 0);	/* gapfill */
 
-	return ((long) pos - (long) buf);
+	return 0;
 }
 
 /***********************************************************************
@@ -401,7 +414,7 @@ int pack_Appointment(struct Appointment *a, unsigned char *buf, int len)
  * Summary:     Fill in the app info structure based on the raw app 
  *		info data
  *
- * Parameters:  None
+ * Parameters:  AppointmentAppInfo_t*, char* to record, record length
  *
  * Returns:     The necessary length of the buffer if record is NULL,
  *		or 0 on error, the length of the data used from the 
@@ -409,8 +422,8 @@ int pack_Appointment(struct Appointment *a, unsigned char *buf, int len)
  *
  ***********************************************************************/
 int
-unpack_AppointmentAppInfo(struct AppointmentAppInfo *ai,
-			  unsigned char *record, int len)
+unpack_AppointmentAppInfo(AppointmentAppInfo_t *ai,
+	unsigned char *record, size_t len)
 {
 	int 	i;
 
@@ -432,7 +445,7 @@ unpack_AppointmentAppInfo(struct AppointmentAppInfo *ai,
  * Summary:     Fill in the raw app info record data based on the app 
  *		info structure
  *
- * Parameters:  None
+ * Parameters:  AppointmentAppInfo*, char* to buffer, buffer length
  *
  * Returns:     The length of the buffer required if record is NULL,
  *		or 0 on error, the length of the data used from the 
@@ -440,8 +453,8 @@ unpack_AppointmentAppInfo(struct AppointmentAppInfo *ai,
  *
  ***********************************************************************/
 int
-pack_AppointmentAppInfo(struct AppointmentAppInfo *ai,
-			unsigned char *record, int len)
+pack_AppointmentAppInfo(AppointmentAppInfo_t *ai,
+	unsigned char *record, size_t len)
 {
 	int 	i;
 	unsigned char *start = record;
@@ -461,3 +474,9 @@ pack_AppointmentAppInfo(struct AppointmentAppInfo *ai,
 
 	return (record - start);
 }
+
+/* vi: set ts=8 sw=4 sts=4 noexpandtab: cin */
+/* Local Variables: */
+/* indent-tabs-mode: t */
+/* c-basic-offset: 8 */
+/* End: */
