@@ -7,7 +7,7 @@
  * under the terms of the GNU Library General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or (at
  * your option) any later version.
- * 
+ *
  * This library is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library
@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU Library General Public License
  * along with this library; if not, write to the Free Software Foundation,
- * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #include <stdio.h>
@@ -28,17 +28,20 @@
  *
  * Function:    free_HiNoteNote
  *
- * Summary:     
+ * Summary:     frees HiNoteNote_t members
  *
- * Parameters:  None
+ * Parameters:  HiNoteNote_t*
  *
- * Returns:     Nothing
+ * Returns:     void
  *
  ***********************************************************************/
-void free_HiNoteNote(struct HiNoteNote *a)
+void
+free_HiNoteNote(HiNoteNote_t *hinote)
 {
-	if (a->text)
-		free(a->text);
+	if (hinote->text != NULL) {
+		free(hinote->text);
+		hinote->text = NULL;
+	}
 }
 
 /***********************************************************************
@@ -47,18 +50,21 @@ void free_HiNoteNote(struct HiNoteNote *a)
  *
  * Summary:     Unpack a HiNote record
  *
- * Parameters:  None
+ * Parameters:  HiNoteNote_t*, char* to buffer, buffer length
  *
- * Returns:     Nothing
+ * Returns:     effective buffer length
  *
  ***********************************************************************/
-int unpack_HiNoteNote(struct HiNoteNote *a, unsigned char *buffer, int len)
+int
+unpack_HiNoteNote(HiNoteNote_t *hinote, unsigned char *buffer, int len)
 {
 	if (len < 3)
 		return 0;
-	a->flags 	= buffer[0];
-	a->level 	= buffer[1];
-	a->text 	= strdup((char *) &buffer[2]);
+
+	hinote->flags 	= buffer[0];
+	hinote->level 	= buffer[1];
+	hinote->text 	= strdup((char *) &buffer[2]);
+
 	return strlen((char *) &buffer[2]) + 3;
 }
 
@@ -68,87 +74,88 @@ int unpack_HiNoteNote(struct HiNoteNote *a, unsigned char *buffer, int len)
  *
  * Summary:     Pack a HiNote record
  *
- * Parameters:  None
+ * Parameters:  HiNoteNote_t*
  *
  * Returns:     Nothing
  *
  ***********************************************************************/
-int pack_HiNoteNote(struct HiNoteNote *a, unsigned char *buffer, int len)
+int
+pack_HiNoteNote(HiNoteNote_t *hinote, unsigned char *buffer, int len)
 {
 	int 	destlen;
 
 	destlen = 3;
-	if (a->text)
-		destlen += strlen(a->text);
+	if (hinote->text)
+		destlen += strlen(hinote->text);
 
 	if (!buffer)
 		return destlen;
 	if (len < destlen)
 		return 0;
 
-	buffer[0] = a->flags;
-	buffer[1] = a->level;
+	buffer[0] = hinote->flags;
+	buffer[1] = hinote->level;
 
-	if (a->text)
-		strcpy((char *) &buffer[2], a->text);
+	if (hinote->text)
+		strcpy((char *) &buffer[2], hinote->text);
 	else {
 		buffer[2] = 0;
 	}
 	return destlen;
 }
 
+
 /***********************************************************************
  *
  * Function:    unpack_HiNoteAppInfo
  *
  * Summary:     Unpack the HiNote AppInfo block
  *
- * Parameters:  None
+ * Parameters:  HiNoteAppInfo_t*, char* to record, record length
  *
- * Returns:     Nothing
+ * Returns:     effective buffer length
  *
  ***********************************************************************/
 int
-unpack_HiNoteAppInfo(struct HiNoteAppInfo *ai, unsigned char *record,
-		     int len)
+unpack_HiNoteAppInfo(HiNoteAppInfo_t *appinfo, unsigned char *record, size_t len)
 {
-	int 	i,	
-		index;
+	int 	i,
+		idx;
 	unsigned char *start;
 
 	start = record;
-	i = unpack_CategoryAppInfo(&ai->category, record, len);
+	i = unpack_CategoryAppInfo(&appinfo->category, record, len);
 	if (!i)
 		return i;
 	record += i;
 	len -= i;
 	if (len < 48)
 		return 0;
-	for (index = 0; i < 48; i++)
-		ai->reserved[i] = *record++;
+	for (idx = 0; i < 48; i++)
+		appinfo->reserved[i] = *record++;
 	return (record - start);
 }
 
+
 /***********************************************************************
  *
  * Function:    pack_HiNoteAppInfo
  *
  * Summary:     Pack the HiNote AppInfo block
  *
- * Parameters:  None
+ * Parameters:  HiNoteAppInfo_t*, char* record, length of record
  *
- * Returns:     Nothing
+ * Returns:     effective record length
  *
  ***********************************************************************/
 int
-pack_HiNoteAppInfo(struct HiNoteAppInfo *ai, unsigned char *record,
-		   int len)
+pack_HiNoteAppInfo(HiNoteAppInfo_t *appinfo, unsigned char *record, size_t len)
 {
 	int 	i,
-		index;
+		idx;
 	unsigned char *start = record;
 
-	i = pack_CategoryAppInfo(&ai->category, record, len);
+	i = pack_CategoryAppInfo(&appinfo->category, record, len);
 	if (i == 0)		/* category pack failed */
 		return 0;
 	if (!record)
@@ -157,8 +164,14 @@ pack_HiNoteAppInfo(struct HiNoteAppInfo *ai, unsigned char *record,
 	len -= i;
 	if (len < 48)
 		return (record - start);
-	for (index = 0; i < 48; i++)
-		*record++ = ai->reserved[i];
+	for (idx = 0; i < 48; i++)
+		*record++ = appinfo->reserved[i];
 
 	return (record - start);
 }
+
+/* vi: set ts=8 sw=4 sts=4 noexpandtab: cin */
+/* Local Variables: */
+/* indent-tabs-mode: t */
+/* c-basic-offset: 8 */
+/* End: */
