@@ -1,4 +1,6 @@
 /*
+ * LICENSE: Public Domain
+ *
  * This code implements the MD5 message-digest algorithm.  The algorithm is
  * due to Ron Rivest.  This code was written by Colin Plumb in 1993, no
  * copyright is claimed.  This code is in the public domain; do with it what
@@ -104,13 +106,13 @@ void MD5Final(UINT8 digest[16], struct MD5Context *ctx)
 	count = 56 - 1 - count;
 
 	if (count < 0) {	/* Padding forces an extra block */
-		memset(p, 0, count + 8);
+		memset(p, 0, (size_t)(count + 8));
 		byteSwap(ctx->in, 16);
 		MD5Transform(ctx->buf, ctx->in);
 		p = (UINT8 *) ctx->in;
 		count = 56;
 	}
-	memset(p, 0, count);
+	memset(p, 0, (size_t)count);
 	byteSwap(ctx->in, 14);
 
 	/* Append length in bits and transform */
@@ -226,3 +228,9 @@ void MD5Transform(UINT32 buf[4], UINT32 const in[16])
 }
 
 #endif
+
+/* vi: set ts=8 sw=4 sts=4 noexpandtab: cin */
+/* Local Variables: */
+/* indent-tabs-mode: t */
+/* c-basic-offset: 8 */
+/* End: */
