@@ -15,18 +15,18 @@
  *
  * You should have received a copy of the GNU Library General Public License
  * along with this library; if not, write to the Free Software Foundation,
- * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. *
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
-#if TIME_WITH_SYS_TIME
+#ifdef TIME_WITH_SYS_TIME
 # include <sys/time.h>
 # include <time.h>
 #else
-# if HAVE_SYS_TIME_H
+# ifdef HAVE_SYS_TIME_H
 #  include <sys/time.h>
 # else
 #  include <time.h>
@@ -40,40 +40,50 @@
 #define DescMaxLength 256
 #define NoteMaxLength 4096
 
+
 /***********************************************************************
  *
  * Function:    free_ToDo
  *
  * Summary:     Free the memory and filehandle from the record alloc. 
  *
- * Parameters:  None
+ * Parameters:  ToDo_t*
  *
- * Returns:     Nothing
+ * Returns:     void
  *
  ***********************************************************************/
-void free_ToDo(struct ToDo *a)
+void
+free_ToDo(ToDo_t *todo)
 {
-	if (a->description)
-		free(a->description);
-	if (a->note)
-		free(a->note);
+
+	if (todo->description != NULL) {
+		free(todo->description);
+		todo->description = NULL;
+	}
+
+	if (todo->note != NULL) {
+		free(todo->note);
+		todo->note = NULL;
+	}
 }
 
+
 /***********************************************************************
  *
  * Function:    unpack_ToDo
  *
  * Summary:     Unpack the ToDo structure into records we can chew on
  *
- * Parameters:  None
+ * Parameters:  ToDo_t*, pi_buffer_t * of buffer, todo type
  *
- * Returns:     Nothing
+ * Returns:     -1 on fail, 0 on success
  *
  ***********************************************************************/
-int unpack_ToDo(struct ToDo *a, unsigned char *buffer, int len)
+int
+unpack_ToDo(ToDo_t *todo, pi_buffer_t *buf, todoType type)
 {
 	unsigned long d;
-	unsigned char *start = buffer;
+	int ofs;
 
 	/* Note: There are possible timezone conversion problems related to
 	   the use of the due member of a struct ToDo. As it is kept in
@@ -89,163 +99,173 @@ int unpack_ToDo(struct ToDo *a, unsigned char *buffer, int len)
 	   appointments.
 	   -- KJA */
 
-	if (len < 3)
-		return 0;
-	d = (unsigned short int) get_short(buffer);
+	if (type != todo_v1)
+		return -1;
+
+	if (buf == NULL || buf->data == NULL || buf->used < 3)
+		return -1;
+
+	d = (unsigned short int) get_short(buf->data);
 	if (d != 0xffff) {
-		a->due.tm_year 	= (d >> 9) + 4;
-		a->due.tm_mon 	= ((d >> 5) & 15) - 1;
-		a->due.tm_mday 	= d & 31;
-		a->due.tm_hour 	= 0;
-		a->due.tm_min 	= 0;
-		a->due.tm_sec 	= 0;
-		a->due.tm_isdst = -1;
-		mktime(&a->due);
-		a->indefinite 	= 0;
+		todo->due.tm_year = (d >> 9) + 4;
+		todo->due.tm_mon = ((d >> 5) & 15) - 1;
+		todo->due.tm_mday = d & 31;
+		todo->due.tm_hour = 0;
+		todo->due.tm_min = 0;
+		todo->due.tm_sec = 0;
+		todo->due.tm_isdst = -1;
+		mktime(&todo->due);
+		todo->indefinite = 0;
 	} else {
-		a->indefinite 	= 1;	/* a->due is invalid */
+		todo->indefinite = 1;	/* todo->due is invalid */
 	}
 
-	a->priority = get_byte(buffer + 2);
-	if (a->priority & 0x80) {
-		a->complete 	= 1;
-		a->priority 	&= 0x7f;
+	todo->priority = get_byte(buf->data + 2);
+	if (todo->priority & 0x80) {
+		todo->complete = 1;
+		todo->priority &= 0x7f;
 	} else {
-		a->complete 	= 0;
+		todo->complete = 0;
 	}
 
-	buffer += 3;
-	len -= 3;
+	ofs = 3;
 
-	if (len < 1)
-		return 0;
-	a->description = strdup((char *) buffer);
+	if (buf->used - ofs < 1)
+		return -1;
 
-	buffer 	+= strlen(a->description) + 1;
-	len 	-= strlen(a->description) + 1;
+	todo->description = strdup((char *) buf->data + ofs);
 
-	if (len < 1) {
-		free(a->description);
-		a->description = 0;
-		return 0;
+	ofs += strlen(todo->description) + 1;
+
+	if (buf->used - ofs < 1) {
+		free(todo->description);
+		todo->description = 0;
+		return -1;
 	}
-	a->note = strdup((char *) buffer);
+	todo->note = strdup((char *) buf->data + ofs);
 
-	buffer 	+= strlen(a->note) + 1;
-	len 	-= strlen(a->note) + 1;
-
-	return (buffer - start);	/* FIXME: return real length */
+	return 0;
 }
 
+
 /***********************************************************************
  *
  * Function:    pack_ToDo
  *
  * Summary:     Pack the ToDo records into a structure
  *
- * Parameters:  None
+ * Parameters:  ToDo_t*, pi_buffer_t *buf of record, record type
  *
- * Returns:     Nothing
+ * Returns:     -1 on error, 0 on success.
  *
  ***********************************************************************/
-int pack_ToDo(struct ToDo *a, unsigned char *buf, int len)
+int
+pack_ToDo(ToDo_t *todo, pi_buffer_t *buf, todoType type)
 {
-	int 	pos,
-		destlen = 3;
+	int pos;
+	size_t destlen = 3;
 
-	if (a->description)
-		destlen += strlen(a->description);
+	if (todo == NULL || buf == NULL)
+		return -1;
+
+	if (type != todo_v1)
+		return -1;
+
+	if (todo->description)
+		destlen += strlen(todo->description);
 	destlen++;
-	if (a->note)
-		destlen += strlen(a->note);
+	if (todo->note)
+		destlen += strlen(todo->note);
 	destlen++;
 
-	if (!buf)
-		return destlen;
-	if (len < destlen)
-		return 0;
+	pi_buffer_expect (buf, destlen);
+	buf->used = destlen;	
 
-	if (a->indefinite) {
-		buf[0] = 0xff;
-		buf[1] = 0xff;
+	if (todo->indefinite) {
+		buf->data[0] = 0xff;
+		buf->data[1] = 0xff;
 	} else {
-		set_short(buf,
-			  ((a->due.tm_year - 4) << 9) | ((a->due.tm_mon +
-							  1) << 5) | a->
+		set_short(buf->data,
+			  ((todo->due.tm_year - 4) << 9) | ((todo->due.tm_mon +
+							  1) << 5) | todo->
 			  due.tm_mday);
 	}
-	buf[2] = a->priority;
-	if (a->complete) {
-		buf[2] |= 0x80;
+	buf->data[2] = todo->priority;
+	if (todo->complete) {
+		buf->data[2] |= 0x80;
 	}
 
 	pos = 3;
-	if (a->description) {
-		strcpy((char *) buf + pos, a->description);
-		pos += strlen(a->description) + 1;
+	if (todo->description) {
+		strcpy((char *) buf->data + pos, todo->description);
+		pos += strlen(todo->description) + 1;
 	} else {
-		buf[pos++] = 0;
+		buf->data[pos++] = 0;
 	}
 
-	if (a->note) {
-		strcpy((char *) buf + pos, a->note);
-		pos += strlen(a->note) + 1;
+	if (todo->note) {
+		strcpy((char *) buf->data + pos, todo->note);
+		pos += strlen(todo->note) + 1;
 	} else {
-		buf[pos++] = 0;
+		buf->data[pos++] = 0;
 	}
 
-	return pos;
+	return 0;
 }
 
+
 /***********************************************************************
  *
  * Function:    unpack_ToDoAppInfo
  *
  * Summary:     Unpack the ToDo AppInfo block from the structure
  *
- * Parameters:  None
+ * Parameters:  ToDoAppInfo_t*, char* to record, record length
  *
- * Returns:     Nothing
+ * Returns:     effective record length
  *
  ***********************************************************************/
 int
-unpack_ToDoAppInfo(struct ToDoAppInfo *ai, unsigned char *record, int len)
+unpack_ToDoAppInfo(ToDoAppInfo_t *appinfo, unsigned char *record, size_t len)
 {
-	int 	i;
+	int i;
 	unsigned char *start = record;
 
-	i = unpack_CategoryAppInfo(&ai->category, record, len);
+	appinfo->type = todo_v1;
+
+	i = unpack_CategoryAppInfo(&appinfo->category, record, len);
 	if (!i)
 		return 0;
 	record += i;
 	len -= i;
 	if (len < 4)
 		return 0;
-	ai->dirty = get_short(record);
+	appinfo->dirty = get_short(record);
 	record += 2;
-	ai->sortByPriority = get_byte(record);
+	appinfo->sortByPriority = get_byte(record);
 	record += 2;
 	return (record - start);
 }
 
+
 /***********************************************************************
  *
  * Function:    pack_ToDoAppInfo
  *
  * Summary:     Pack the AppInfo block/record back into the structure
  *
- * Parameters:  None
+ * Parameters:  ToDoAppInfo_t*, char* to record, record length
  *
- * Returns:     Nothing
+ * Returns:     effective buffer length
  *
  ***********************************************************************/
 int
-pack_ToDoAppInfo(struct ToDoAppInfo *ai, unsigned char *record, int len)
+pack_ToDoAppInfo(ToDoAppInfo_t *appinfo, unsigned char *record, size_t len)
 {
-	int 	i;
+	int i;
 	unsigned char *start = record;
 
-	i = pack_CategoryAppInfo(&ai->category, record, len);
+	i = pack_CategoryAppInfo(&appinfo->category, record, len);
 	if (!record)
 		return i + 4;
 	if (!i)
@@ -254,10 +274,16 @@ pack_ToDoAppInfo(struct ToDoAppInfo *ai, unsigned char *record, int len)
 	len -= i;
 	if (len < 4)
 		return 0;
-	set_short(record, ai->dirty);
-	set_byte(record + 2, ai->sortByPriority);
+	set_short(record, appinfo->dirty);
+	set_byte(record + 2, appinfo->sortByPriority);
 	set_byte(record + 3, 0);	/* gapfill */
 	record += 4;
 
 	return (record - start);
 }
+
+/* vi: set ts=8 sw=4 sts=4 noexpandtab: cin */
+/* Local Variables: */
+/* indent-tabs-mode: t */
+/* c-basic-offset: 8 */
+/* End: */
