@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU Library General Public License
  * along with this library; if not, write to the Free Software Foundation,
- * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. *
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -24,37 +24,24 @@
 #endif
 
 #include <stdio.h>
+#include <stdlib.h>
 #include <ctype.h>
 #include <math.h>
-
-#ifdef NeXT
-# include <stdlib.h>
-# include <assert.h>
-#endif
-
-#ifdef WIN32
-#include <winsock.h>
-#include <assert.h>
-#else
-#ifndef HAVE_INET_ATON
-# include <sys/param.h>
-#ifdef __EMX__
-# include <sys/types.h>
-# define INCL_DOSFILEMGR	/* File System values */
-# define INCL_MISC
-# include <os2.h>
-#endif
-# include <sys/types.h>
-# include <netinet/in.h>
-# include <arpa/inet.h>
-# include <ctype.h>
-#endif
-#endif
-# include <string.h>
+#include <sys/types.h>
+#include <netinet/in.h>
+#include <arpa/inet.h>
+#include <ctype.h>
+#include <string.h>
 
 #include "pi-debug.h"
 #include "pi-source.h"
-#include "pi-socket.h"
+
+void pi_timeout_to_timespec(int timeout, struct timespec *ts);
+void get_pilot_rate(int *establishrate, int *establishhighrate);
+int pi_timespec_to_timeout(const struct timespec *ts);
+int pi_timeout_expired(const struct timespec *ts);
+size_t palm_strftime(char *s, size_t max, const char *fmt, 
+        const struct tm *tm);
 
 /* this routine ruthlessly stolen verbatim from Brian J. Swetland */
 
@@ -86,59 +73,50 @@ int crc16(unsigned char *ptr, int count)
 	return (crc & 0xFFFF);
 }
 
-#ifndef HAVE_STRDUP
-/***********************************************************************
- *
- * Function:    strdup
- *
- * Summary:     Duplicate a string
- *
- * Parameters:  None
- *
- * Returns:     String or NULL
- *
- ***********************************************************************/
-char *strdup(const char *string)
+void get_pilot_rate(int *establishrate, int *establishhighrate)
 {
-	size_t 	length;
-	char 	*result;
+	/* Default PADP connection rate */
+	char *rate_env = getenv("PILOTRATE");
+	if (rate_env) {
+		/* Establish high rate */
+		if (rate_env[0] == 'H') {
+			*establishrate = atoi(rate_env + 1);
+			*establishhighrate = 1;
+		} else {
+			*establishrate = atoi(rate_env);
+			*establishhighrate = 0;
+		}
+	}
+	else {
+		*establishrate = -1;
+	}
+}
 
-	assert(string != NULL);
+#ifndef HAVE_STRDUP
+char *strdup(const char *s)
+{
+        char *result;
+        size_t size = strlen(s) + 1;
+   
+        if (!(result = malloc(size))) {
+                return NULL;
+        }
 
-	length = strlen(string) + 1;
-	result = malloc(length);
+        memcpy(result, s, size);
 
-	if (result == NULL)
-		return NULL;
-
-	memcpy(result, string, length);
-
-	return result;
+        return result;
 }
 #endif
 
-#ifndef HAVE_PUTENV
-
 /* Borrowed from GNU sh-utils, and then probably from GNU libc */
-
+#ifndef HAVE_PUTENV
 #if HAVE_GNU_LD
-# define environ __environ
+	# define environ __environ
 #else
-extern char **environ;
+	extern char **environ;
 #endif
 
-/***********************************************************************
- *
- * Function:    putenv
- *
- * Summary:     Put STRING, which is of the form "NAME=VALUE", in the
- *		environment
- *
- * Parameters:  None
- *
- * Returns:     0 for success, nonzero if any errors occur
- *
- ***********************************************************************/
+/* Put STRING, which is of the form "NAME=VALUE", in the environment */
 int putenv(const char *string)
 {
 	const char 	*const name_end = strchr(string, '=');
@@ -191,6 +169,25 @@ int putenv(const char *string)
 }
 #endif
 
+#ifdef OS2
+/* Replacement version of getenv(), because the one in the EMX 0.9c, fix03
+   dist appears to be busted when called from inside a DLL. (MJJ) */
+char *getenv(const char *envar)
+{
+	APIRET 	rc;
+	unsigned char *envstring;
+
+	/* just call the OS/2 function directly */
+	rc = DosScanEnv(envar, &envstring);
+	if (rc)
+		return NULL;
+	else
+		return envstring;
+}
+#endif
+
+
+
 #ifndef HAVE_INET_ATON
 /***********************************************************************
  *
@@ -298,66 +295,35 @@ char *printlong(unsigned long val)
 
 unsigned long makelong(char *c)
 {
-	int 	l 	= strlen(c);
+	int 	l = strlen(c);
 	char 	c2[4];
 
 	if (l >= 4)
 		return get_long(c);
 	memset(c2, ' ', 4);
-	memcpy(c2, c, l);
+	memcpy(c2, c, (size_t)l);
 	return get_long(c2);
 }
 
-void dumpline(const unsigned char *buf, int len, int addr)
-{
-	int 	i;
-
-	pi_log(PI_DBG_ALL, PI_DBG_LVL_NONE, "  %.4x  ", addr);
-
-	for (i = 0; i < 16; i++) {
-
-		if (i < len)
-			pi_log(PI_DBG_ALL, PI_DBG_LVL_NONE, "%.2x ",
-			       0xff & (unsigned int) buf[i]);
-		else
-			pi_log(PI_DBG_ALL, PI_DBG_LVL_NONE, "   ");
-	}
-
-	pi_log(PI_DBG_ALL, PI_DBG_LVL_NONE, "  ");
-
-	for (i = 0; i < len; i++) {
-		if (isprint(buf[i]) && (buf[i] >= 32) && (buf[i] <= 126))
-			pi_log(PI_DBG_ALL, PI_DBG_LVL_NONE, "%c", buf[i]);
-		else
-			pi_log(PI_DBG_ALL, PI_DBG_LVL_NONE, ".");
-	}
-	pi_log(PI_DBG_ALL, PI_DBG_LVL_NONE, "\n");
-}
-
-void dumpdata(const unsigned char *buf, int len)
-{
-	int 	i;
-
-	for (i = 0; i < len; i += 16)
-		dumpline(buf + i, ((len - i) > 16) ? 16 : len - i, i);
-}
-
 double get_float(void *buffer)
 {
 	unsigned char *buf = buffer;
-	int 	exp 	= get_sshort(buf + 4),
-		sign 	= get_byte(buf + 6);
 	unsigned long frac = get_long(buf);
 
-	return ldexp(sign ? (double) frac : -(double) frac, exp);
+	int 	expr	= get_sshort(buf + 4),
+		sign 	= get_byte(buf + 6);
+
+        /* if (sign) f = frac; else f = -frac; return ldexp(f, exp); */
+        return ldexp(sign ? (double) frac : -(double) frac, expr);
 }
 
 void set_float(void *buffer, double value)
 {
-	int 	exp, 
+	int 	expr, 
 		sign;
 	unsigned char *buf = buffer;
 	unsigned long frac;
+	double r;
 
 	/* Take absolute */
 	if (value < 0) {
@@ -367,12 +333,13 @@ void set_float(void *buffer, double value)
 		sign 	= 0xFF;
 
 	/* Convert mantissa to 32-bit integer, and take exponent */
-	frac = (unsigned long) ldexp(frexp(value, &exp), 32);
-	exp -= 32;
+	r = ldexp(frexp(value, &expr), 32);
+	frac = (unsigned long)r;
+	expr -= 32;
 
 	/* Store values in buffer */
 	set_long(buf, frac);
-	set_sshort(buf + 4, exp);
+	set_sshort(buf + 4, expr);
 	set_byte(buf + 6, sign);
 	set_byte(buf + 7, 0);
 }
@@ -400,21 +367,44 @@ int compareTm(struct tm *a, struct tm *b)
 	return date;
 }
 
-#ifdef OS2
-
-/* Replacement version of getenv(), because the one in the EMX 0.9c, fix03
-   dist appears to be busted when called from inside a DLL. (MJJ) */
-char *getenv(const char *envar)
+void pi_timeout_to_timespec(int timeout, struct timespec *ts)
 {
-	APIRET 	rc;
-	unsigned char *envstring;
-
-	/* just call the OS/2 function directly */
-	rc = DosScanEnv(envar, &envstring);
-	if (rc)
-		return NULL;
-	else
-		return envstring;
+	/* convert a timeout value (in milliseconds) to an absolute timespec */
+	struct timeval now;
+	gettimeofday(&now, NULL);
+	ts->tv_sec = now.tv_sec + (long)(timeout / 1000);
+	ts->tv_nsec = (now.tv_usec + ((long)timeout % 1000) * 1000) * 1000;
+	if (ts->tv_nsec >= 1000000000) {
+		ts->tv_nsec -= 1000000000;
+		ts->tv_sec++;
+	}
 }
 
-#endif
+int pi_timespec_to_timeout(const struct timespec *ts)
+{
+	/* convert an absolute timespec to a timeout value (in milliseconds) from now
+	* returns a negative if the timeout expired already 
+	*/
+	struct timeval now;
+	gettimeofday(&now, NULL);
+	return (int)(((double)ts->tv_sec * 1000.0 + (double)ts->tv_nsec / 1000000.0) -
+		     ((double)now.tv_sec * 1000.0 + (double)now.tv_usec / 1000.0));
+}
+
+int pi_timeout_expired(const struct timespec *ts)
+{
+	return pi_timespec_to_timeout(ts) <= 0;
+}
+
+/* Fix some issues with some locales reporting 2 or 4 digit years */
+size_t palm_strftime(char *s, size_t max, const char *fmt,
+        const struct tm *tm) {
+
+        return strftime(s, max, fmt, tm);
+}
+
+/* vi: set ts=8 sw=4 sts=4 noexpandtab: cin */
+/* Local Variables: */
+/* indent-tabs-mode: t */
+/* c-basic-offset: 8 */
+/* End: */
