@@ -13,50 +13,31 @@
  *
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
- * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  *
  */
 
-#include "getopt.h"
 #include <stdio.h>
-#include <stdlib.h>
-#include <unistd.h>
 #include <netinet/in.h>
 
 #include "pi-source.h"
-#include "pi-socket.h"
 #include "pi-syspkt.h"
 #include "pi-dlp.h"
 #include "pi-header.h"
+#include "pi-userland.h"
 
-/* Declare prototypes */
-static void display_help(char *progname);
-void print_splash(char *progname);
-int pilot_connect(char *port);
-
-struct option options[] = {
-	{"port",        required_argument, NULL, 'p'},
-	{"help",        no_argument,       NULL, 'h'},
-	{"version",     no_argument,       NULL, 'v'},
-	{"get",         no_argument,       NULL, 'g'},
-	{"set",         no_argument,       NULL, 's'},
-	{NULL,          0,                 NULL, 0}
-};
-
-static const char *optstring = "p:hvgs";
-
-static void *GetClip(int socket, int type, int *length)
+static void *GetClip(int sock, int type, int *length)
 {
-	int 	l,
+	int 	len,
 		err;
 	struct 	RPC_params p;
 	unsigned long handle, ptr;
 	void 	*buffer;
 
 	/* ClipboardGetItem */
-	PackRPC(&p, 0xA10C, RPC_PtrReply, RPC_Byte(type), RPC_ShortPtr(&l),
+	PackRPC(&p, 0xA10C, RPC_PtrReply, RPC_Byte(type), RPC_ShortPtr(&len),
 		RPC_End);
-	err = dlp_RPC(socket, &p, &handle);
+	err = dlp_RPC(sock, &p, &handle);
 	if (err)
 		return 0;
 
@@ -65,29 +46,29 @@ static void *GetClip(int socket, int type, int *length)
 
 	/* MemHandleLock */
 	PackRPC(&p, 0xA021, RPC_PtrReply, RPC_Long(handle), RPC_End);
-	err = dlp_RPC(socket, &p, &ptr);
+	err = dlp_RPC(sock, &p, &ptr);
 
 	if (err)
 		return 0;
 
-	buffer = malloc(l);
+	buffer = malloc(len);
 
 	/* MemMove */
-	PackRPC(&p, 0xA026, RPC_IntReply, RPC_Ptr(buffer, l),
-		RPC_Long(ptr), RPC_Long(l), RPC_End);
-	err = dlp_RPC(socket, &p, 0);
+	PackRPC(&p, 0xA026, RPC_IntReply, RPC_Ptr(buffer, len),
+		RPC_Long(ptr), RPC_Long(len), RPC_End);
+	err = dlp_RPC(sock, &p, 0);
 
 	/* MemHandleUnlock */
 	PackRPC(&p, 0xA022, RPC_IntReply, RPC_Long(handle), RPC_End);
-	err = dlp_RPC(socket, &p, 0);
+	err = dlp_RPC(sock, &p, 0);
 
 	if (length)
-		*length = l;
+		*length = len;
 
 	return buffer;
 }
 
-static int SetClip(int socket, int type, void *data, int length)
+static int SetClip(int sock, int type, void *data, int length)
 {
 	int 	err;
 	char 	*b = data;
@@ -96,7 +77,7 @@ static int SetClip(int socket, int type, void *data, int length)
 
 	/* MemHandleNew */
 	PackRPC(&p, 0xA01E, RPC_PtrReply, RPC_Long(length), RPC_End);
-	err = dlp_RPC(socket, &p, &handle);
+	err = dlp_RPC(sock, &p, &handle);
 	if (err)
 		return 0;
 
@@ -105,7 +86,7 @@ static int SetClip(int socket, int type, void *data, int length)
 
 	/* MemHandleLock */
 	PackRPC(&p, 0xA021, RPC_PtrReply, RPC_Long(handle), RPC_End);
-	err = dlp_RPC(socket, &p, &ptr);
+	err = dlp_RPC(sock, &p, &ptr);
 
 	if (err)
 		return 0;
@@ -113,86 +94,85 @@ static int SetClip(int socket, int type, void *data, int length)
 	/* MemMove */
 	PackRPC(&p, 0xA026, RPC_IntReply, RPC_Long(ptr),
 		RPC_Ptr(b, length), RPC_Long(length), RPC_End);
-	err = dlp_RPC(socket, &p, 0);
+	err = dlp_RPC(sock, &p, 0);
 
 	length--;
 
 	/* ClipboardAddItem */
 	PackRPC(&p, 0xA10A, RPC_IntReply, RPC_Byte(type), RPC_Long(ptr),
 		RPC_Short(length), RPC_End);
-	err = dlp_RPC(socket, &p, 0);
+	err = dlp_RPC(sock, &p, 0);
 
 	/* MemPtrFree */
 	PackRPC(&p, 0xA012, RPC_IntReply, RPC_Long(ptr), RPC_End);
-	err = dlp_RPC(socket, &p, 0);
+	err = dlp_RPC(sock, &p, 0);
 	return 1;
 }
 
-static void display_help(char *progname)
-{
-	printf("   Get or Set the Palm Clipboard contents from STDOUT/STDIN\n\n");
-	printf("   Usage: %s -p <port> -g | -s <value>\n\n", progname);
-	printf("   Options:\n");
-	printf("     -p, --port <port>       Use device file <port> to communicate with Palm\n");
-	printf("     -h, --help              Display help information for %s\n", progname);
-	printf("     -v, --version           Display %s version information\n", progname);
-	printf("     -g, --get               Get the contents of the clipboard\n");
-	printf("     -s, --set <value>       Set the value <value> in the clipboard\n\n");
-	printf("   Examples: %s -p /dev/pilot -g\n", progname);
-	printf("             %s -p /dev/pilot -s \"Put this in the clipboard\"\n\n", progname);
 
-	return;
-}
-
-int main(int argc, char *argv[])
+int main(int argc, const char *argv[])
 {
 	int 	c,		/* switch */
 		sd 		= -1,
-		getset          = -1,
 		ret;
-	char 	buffer[0xffff],
-		*progname 	= argv[0],
-		*port		= NULL;
 
-	while ((c = getopt_long(argc, argv, optstring, options, NULL)) != -1) {
+	enum { mode_none, mode_set='s', mode_get='g' } mode = mode_none;
+
+	char 	buffer[0xffff];
+
+	poptContext po;
+
+	struct poptOption options[] = {
+		USERLAND_RESERVED_OPTIONS
+	        {"get",		'g', POPT_ARG_NONE, NULL,   'g', 
+			"Print the contents of the clipboard on STDOUT", NULL},
+        	{"set",		's', POPT_ARG_NONE, NULL,   's', 
+			"Set the value in the clipboard from STDIN", NULL},
+                POPT_AUTOHELP
+                POPT_TABLEEND
+	};
+
+	po = poptGetContext("pilot-clip", argc, argv, options, 0);
+	poptSetOtherOptionHelp(po,"\n\n"
+	"   Get or Set the Palm Clipboard contents from STDOUT/STDIN.\n"
+	"   Must provide one of -g or -s.\n\n");
+
+	if (argc < 2) {
+		poptPrintUsage(po,stderr,0);
+		return 1;
+	}
+
+	while ((c = poptGetNextOpt(po)) >= 0) {
 		switch (c) {
-
-		case 'h':
-			display_help(progname);
-			return 0;
-		case 'v':
-			print_splash(progname);
-			return 0;
-		case 'p':
-			port = optarg;
-			break;
-		case 'g':
-			getset = 0;
-			break;
 		case 's':
-			getset = 1;
+		case 'g':
+			if (mode != mode_none) {
+				fprintf(stderr,
+					"   ERROR: --get or --set, not both.\n");
+				return 1;
+			}
+			mode = c;
 			break;
 		default:
-			display_help(progname);
-			return 0;
+			fprintf(stderr,"   ERROR: Unhandled option %d.\n", c);
+			return 1;
 		}
 	}
-	if (getset < 0) {
-		display_help(progname);
-		fprintf(stderr, "ERROR: You must specify whether to "
+	if (mode_none == mode) {
+		fprintf(stderr, "   ERROR: You must specify whether to "
 				"\"Get\" or \"Set\" the clipboard\n");
 		return -1;
 	}
-	
-	sd = pilot_connect(port);
+
+	sd = plu_connect();
 	if (sd < 0)
-		goto error;	
+		goto error;
 
 	if (dlp_OpenConduit(sd) < 0)
 		goto error_close;
 
 
-	if (getset == 1) {
+	if (mode == mode_set) {
 		ret = read(fileno(stdin), buffer, 0xffff);
 		if (ret >= 0) {
 			buffer[ret++] = 0;
@@ -201,7 +181,7 @@ int main(int argc, char *argv[])
 		}
 	} else {
 		char *b;
-		
+
 		ret = 0;
 		b = GetClip(sd, 0, &ret);
 		if (b == NULL)
@@ -217,7 +197,7 @@ int main(int argc, char *argv[])
 
 error_close:
 	pi_close(sd);
-	
+
 error:
-	return -1;	
+	return -1;
 }
