@@ -16,11 +16,10 @@
  *
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
- * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  *
  */
 
-#include "getopt.h"
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -28,26 +27,8 @@
 
 #include "pi-header.h"
 #include "pi-source.h"
-#include "pi-socket.h"
-#include "pi-dlp.h"
 #include "pi-file.h"
-
-struct option options[] = {
-	{"help",        no_argument,       NULL, 'h'},
-	{"version",     no_argument,       NULL, 'v'},
-	{"header",      no_argument,       NULL, 'H'},
-	{"appinfo",     no_argument,       NULL, 'a'},
-	{"sortinfo",    no_argument,       NULL, 's'},
-	{"list",        no_argument,       NULL, 'l'},
-	{"record",      required_argument, NULL, 'r'},
-	{"record-res",  required_argument, NULL, 'R'},
-	{"dump",        no_argument,       NULL, 'd'},
-	{"dump-res",    no_argument,       NULL, 'D'},
-	{NULL,          0,                 NULL, 0}
-};
-
-static const char *optstring = "hvHaslr:R:dD";
-
+#include "pi-userland.h"
 
 /***********************************************************************
  *
@@ -72,6 +53,7 @@ static char *iso_time_str(time_t t)
 	return (buf);
 }
 
+
 /***********************************************************************
  *
  * Function:    dump
@@ -110,6 +92,7 @@ static void dump(void *buf, int n)
 	}
 }
 
+
 /***********************************************************************
  *
  * Function:    dump_header
@@ -123,46 +106,47 @@ static void dump(void *buf, int n)
  ***********************************************************************/
 static void dump_header(struct pi_file *pf, struct DBInfo *ip)
 {
-	printf("name: \"%s\"\n", ip->name);
-	printf("flags: 0x%x", ip->flags);
+	printf("Name..........: %s\n", ip->name);
+	printf("flags.........: 0x%x", ip->flags);
 	if (ip->flags & dlpDBFlagNewer)
 		printf(" NEWER");
-	
+
 	if (ip->flags & dlpDBFlagReset)
 		printf(" RESET");
-	
+
 	if (ip->flags & dlpDBFlagResource)
 		printf(" RESOURCE");
-	
+
 	if (ip->flags & dlpDBFlagReadOnly)
 		printf(" READ_ONLY");
-	
+
 	if (ip->flags & dlpDBFlagAppInfoDirty)
 		printf(" APP-INFO-DIRTY");
-	
+
 	if (ip->flags & dlpDBFlagBackup)
 		printf(" BACKUP");
-	
+
 	if (ip->flags & dlpDBFlagCopyPrevention)
 		printf(" COPY-PREVENTION");
-	
+
 	if (ip->flags & dlpDBFlagStream)
 		printf(" STREAM");
-	
+
 	if (ip->flags & dlpDBFlagOpen)
 		printf(" OPEN");
-	
+
 	printf("\n");
-	printf("version: %d\n", ip->version);
-	printf("creation_time: %s\n", iso_time_str(ip->createDate));
-	printf("modified_time: %s\n", iso_time_str(ip->modifyDate));
-	printf("backup_time: %s\n", iso_time_str(ip->backupDate));
-	printf("modification_number: %ld\n", ip->modnum);
-	printf("type: '%s', ", printlong(ip->type));
-	printf("creator: '%s'\n", printlong(ip->creator));
+	printf("version.......: %d\n", ip->version);
+	printf("creation_time.: %s\n", iso_time_str(ip->createDate));
+	printf("modified_time.: %s\n", iso_time_str(ip->modifyDate));
+	printf("backup_time...: %s\n", iso_time_str(ip->backupDate));
+	printf("mod_number....: %ld\n", ip->modnum);
+	printf("type..........: %s\n", printlong(ip->type));
+	printf("CreatorID.....: %s\n", printlong(ip->creator));
 	printf("\n");
 }
 
+
 /***********************************************************************
  *
  * Function:    dump_app_info
@@ -176,19 +160,17 @@ static void dump_header(struct pi_file *pf, struct DBInfo *ip)
  ***********************************************************************/
 static void dump_app_info(struct pi_file *pf, struct DBInfo *ip)
 {
-	int 	app_info_size;
+	size_t 	app_info_size;
 	void 	*app_info;
 
-	if (pi_file_get_app_info(pf, &app_info, &app_info_size) < 0) {
-		printf("can't get app_info\n\n");
-		return;
-	}
+	pi_file_get_app_info(pf, &app_info, &app_info_size);
 
-	printf("app_info_size %d\n", app_info_size);
+	printf("app_info_size %zu\n", app_info_size);
 	dump(app_info, app_info_size);
 	printf("\n");
 }
 
+
 /***********************************************************************
  *
  * Function:    dump_sort_info
@@ -202,20 +184,18 @@ static void dump_app_info(struct pi_file *pf, struct DBInfo *ip)
  ***********************************************************************/
 static void dump_sort_info(struct pi_file *pf, struct DBInfo *ip)
 {
-	int 	sort_info_size;
+	size_t 	sort_info_size;
 	void 	*sort_info;
 
 
-	if (pi_file_get_sort_info(pf, &sort_info, &sort_info_size) < 0) {
-		printf("can't get sort_info\n\n");
-		return;
-	}
+	pi_file_get_sort_info(pf, &sort_info, &sort_info_size);
 
-	printf("sort_info_size %d\n", sort_info_size);
+	printf("sort_info_size %zu\n", sort_info_size);
 	dump(sort_info, sort_info_size);
 	printf("\n");
 }
 
+
 /***********************************************************************
  *
  * Function:    list_records
@@ -232,9 +212,9 @@ static void list_records(struct pi_file *pf, struct DBInfo *ip, int filedump, in
 	int 	attrs,
 		cat,
 		entnum,		/* Number of the entry record */
-		id,
-		nentries, 	/* Number of entries in the list */
-		size;
+		id_,
+		nentries; 	/* Number of entries in the list */
+	size_t	size;
 	void 	*buf;
 	unsigned long type, uid;
 
@@ -245,12 +225,12 @@ static void list_records(struct pi_file *pf, struct DBInfo *ip, int filedump, in
 		printf("index\tsize\ttype\tid\n");
 		for (entnum = 0; entnum < nentries; entnum++) {
 			if (pi_file_read_resource
-			    (pf, entnum, &buf, &size, &type, &id) < 0) {
+			    (pf, entnum, &buf, &size, &type, &id_) < 0) {
 				printf("error reading %d\n\n", entnum);
 				return;
 			}
-			printf("%d\t%d\t%s\t%d\n", entnum, size,
-			       printlong(type), id);
+			printf("%d\t%zu\t%s\t%d\n", entnum, size,
+			       printlong(type), id_);
 			if (verbose) {
 				dump(buf, size);
 				printf("\n");
@@ -259,7 +239,7 @@ static void list_records(struct pi_file *pf, struct DBInfo *ip, int filedump, in
 					char name[64];
 
 					sprintf(name, "%4s%04x.bin",
-						printlong(type), id);
+						printlong(type), id_);
 					fp = fopen(name, "w");
 					fwrite(buf, size, 1, fp);
 					fclose(fp);
@@ -276,7 +256,7 @@ static void list_records(struct pi_file *pf, struct DBInfo *ip, int filedump, in
 				printf("error reading %d\n\n", entnum);
 				return;
 			}
-			printf("%d\t%d\t0x%x\t%d\t0x%lx\n", entnum, size,
+			printf("%d\t%zu\t0x%x\t%d\t0x%lx\n", entnum, size,
 			       attrs, cat, uid);
 			if (verbose) {
 				dump(buf, size);
@@ -288,6 +268,7 @@ static void list_records(struct pi_file *pf, struct DBInfo *ip, int filedump, in
 	printf("\n");
 }
 
+
 /***********************************************************************
  *
  * Function:    dump_record
@@ -303,9 +284,9 @@ static void dump_record(struct pi_file *pf, struct DBInfo *ip, char *rkey, int f
 {
 	int 	attrs,
 		cat,
-		id,
-		record,
-		size;
+		id_,
+		record;
+	size_t	size;
 	void 	*buf;
 	unsigned long type;
 	unsigned long uid;
@@ -316,31 +297,32 @@ static void dump_record(struct pi_file *pf, struct DBInfo *ip, char *rkey, int f
 		printf("index\tsize\ttype\tid\n");
 		if (sscanf(rkey, "%d", &record) == 1) {
 			if (pi_file_read_resource
-			    (pf, record, &buf, &size, &type, &id) < 0) {
+			    (pf, record, &buf, &size, &type, &id_) < 0) {
 				printf("error reading resource #%d\n\n",
 				       record);
 				return;
 			}
 		} else {
 			type = makelong(rkey);
-			id = 0;
-			sscanf(&rkey[4], "%x", &id);
+			id_ = 0;
+			sscanf(&rkey[4], "%d", &id_);
 			if (pi_file_read_resource_by_type_id
-			    (pf, type, id, &buf, &size, &record) < 0) {
+			    (pf, type, id_, &buf, &size, &record) < 0) {
 				printf
 				    ("error reading resource %s' #%d (0x%x)\n\n",
-				     printlong(type), id, id);
+				     printlong(type), id_, id_);
 				return;
 			}
 		}
-		printf("%d\t%d\t%s\t%d\n", record, size, printlong(type),
-		       id);
+
+		printf("%d\t%zu\t%s\t%d\n", record, size, printlong(type),
+		       id_);
 		dump(buf, size);
 		if (filedump) {
 			FILE *fp;
 			char name[64];
 
-			sprintf(name, "%4s%04x.bin", printlong(type), id);
+			sprintf(name, "%4s%04x.bin", printlong(type), id_);
 			fp = fopen(name, "w");
 			fwrite(buf, size, 1, fp);
 			fclose(fp);
@@ -368,7 +350,7 @@ static void dump_record(struct pi_file *pf, struct DBInfo *ip, char *rkey, int f
 				return;
 			}
 		}
-		printf("%d\t%d\t0x%x\t%d\t0x%lx\n", record, size, attrs,
+		printf("%d\t%zu\t0x%x\t%d\t0x%lx\n", record, size, attrs,
 		       cat, uid);
 		dump(buf, size);
 	}
@@ -376,110 +358,103 @@ static void dump_record(struct pi_file *pf, struct DBInfo *ip, char *rkey, int f
 	printf("\n");
 }
 
-static void display_help(char *progname)
-{
-	printf("   Dump application and header information from your local PRC/PDB files\n\n");
-	printf("   Usage: %s [options] file\n", progname);
-	printf("   Options:\n");
-	printf("     -H --header       Dump the header of the database(s)\n");
-	printf("     -a --appinfo      Dump app_info segment of the database(s)\n");
-	printf("     -s --sortinfo     Dump sort_info block of database(s)\n");
-	printf("     -l                List all records in the database(s)\n");
-	printf("     -r <num>          Dump a record (<num> is an index, or eg 'code0' or a uid '1234')\n");
-	printf("     -R <num>          As above but also dump resources to files\n");
-	printf("     -d --dump         Dump all data and all records, very verbose\n");
-	printf("     -D --dump-res     As above but also dump resources to files\n");
-	printf("     -h --help         Display this information\n");
-	printf("     -v, --version     Display version information\n\n");
-	printf("        Examples: %s -l Foo.prc\n", progname);
-	printf("                  %s -H -a Bar.pdb\n\n", progname);
-
-	exit(0);
-}
 
 
-int main(int argc, char **argv)
+
+int main(int argc, const char **argv)
 {
 	int 	c,		/* switch */
 		hflag 		= 0,
 		aflag 		= 0,
 		sflag 		= 0,
 		dflag 		= 0,
-		lflag 		= 0, 
+		lflag 		= 0,
 		rflag 		= 0,
 		filedump 	= 0;
-	char 	*name,
-		*rkey           = NULL,
-		*progname 	= argv[0];
+
+	char *rkey           = NULL;
+
+	const char **rargv = NULL;
+
 	struct 	pi_file *pf;
 	struct 	DBInfo info;
 
-	while ((c = getopt_long(argc, argv, optstring, options, NULL)) != -1) {
-		switch (c) {
-		case 'h':
-			display_help(progname);
-			exit(0);
-		case 'v':
-			print_splash(progname);
-			exit(0);
-		case 'H':
-			hflag = 1;
-			break;
-		case 'a':
-			aflag = 1;
-			break;
-		case 's':
-			sflag = 1;
-			break;
-		case 'D':
-			filedump = 1;
-		case 'd':
-			dflag = 1;
-			break;
-		case 'l':
-			lflag = 1;
-			break;
-		case 'R':
-			filedump = 1;
-		case 'r':
-			rflag = 1;
-			rkey = optarg;
-			break;
+	poptContext po;
+
+	struct poptOption options[] = {
+		USERLAND_RESERVED_OPTIONS
+        	{"header",	'H', POPT_ARG_NONE, &hflag, 0, "Dump the header of the database(s)"},
+	        {"appinfo",	'a', POPT_ARG_NONE, &aflag, 0, "Dump app_info segment of the database(s)"},
+        	{"sortinfo",	's', POPT_ARG_NONE, &sflag, 0, "Dump sort_info block of database(s)"},
+	        {"list",	'l', POPT_ARG_NONE, &lflag, 0, "List all records in the database(s)"},
+        	{"record",	'r', POPT_ARG_STRING, &rkey,  0, "Dump a record by index ('code0') or uid ('1234')"},
+	        {"to-file",	'f', POPT_ARG_NONE, &filedump,  0, "Same as above but also dump records to files"},
+        	{"dump",	'd', POPT_ARG_NONE, &dflag, 0, "Dump all data and all records, very verbose"},
+        	  POPT_TABLEEND
+	};
+
+	po = poptGetContext("pilot-file", argc, argv, options, 0);
+	poptSetOtherOptionHelp(po,"<filename> ...\n\n"
+	"   Dump application and header information from your local PRC/PDB files\n\n"
+	"   Example arguments:\n"
+	"      -l Foo.prc\n"
+	"      -H -a Bar.pdb\n\n");
+
+	plu_popt_alias(po,"dump-rec",0,"--bad-option --to-file --record");
+	plu_popt_alias(po,NULL,'R',"--bad-option --to-file --record");
+	plu_popt_alias(po,"dump-res",0,"--bad-option --to-file --dump");
+	plu_popt_alias(po,NULL,'D',"--bad-option --to-file --dump");
+
+	if (argc < 2) {
+		poptPrintUsage(po,stderr,0);
+		return 1;
+	}
+
+	while ((c = poptGetNextOpt(po)) >= 0) {
+		fprintf(stderr,"   ERROR: Unhandled option %d.\n",c);
+		return 1;
+	}
+
+	if (c < -1) {
+		plu_badoption(po,c);
+	}
+
+	if (rkey) {
+		rflag = 1;
+	}
+
+	rargv = poptGetArgs(po);
+	if (!rargv || !rargv[0]) {
+		fprintf(stderr,"   ERROR: Must provide one or more filenames.\n");
+		return 1;
+	}
+
+	while (*rargv) {
+		const char *name = *rargv++;
+
+		if ((pf = pi_file_open(name)) == NULL) {
+			fprintf(stderr, "   ERROR: Can't open '%s' Does '%s' exist?\n\n",
+				name, name);
+			continue;
 		}
+
+		pi_file_get_info(pf, &info);
+
+		if (hflag || dflag)
+			dump_header(pf, &info);
+
+		if (aflag || dflag)
+			dump_app_info(pf, &info);
+
+		if (sflag || dflag)
+			dump_sort_info(pf, &info);
+
+		if (lflag || dflag)
+			list_records(pf, &info, filedump, dflag);
+
+		if (rflag)
+			dump_record(pf, &info, rkey, filedump);
 	}
 
-	if (optind > 1) {
-		name = argv[optind];
-	} else {
-		display_help(progname);
-		fprintf(stderr, "ERROR: You must specify a file\n");
-		return -1;
-	}
-	
-	if ((pf = pi_file_open(name)) == NULL) {
-		fprintf(stderr, "can't open %s\n", name);
-		return -1;
-	}
-
-	if (pi_file_get_info(pf, &info) < 0) {
-		fprintf(stderr, "can't get info\n\n");
-		return -1;
-	}
-
-	if (hflag || dflag)
-		dump_header(pf, &info);
-
-	if (aflag || dflag)
-		dump_app_info(pf, &info);
-
-	if (sflag || dflag)
-		dump_sort_info(pf, &info);
-
-	if (lflag || dflag)
-		list_records(pf, &info, filedump, dflag);
-
-	if (rflag)
-		dump_record(pf, &info, rkey, filedump);
-
 	return 0;
 }
