@@ -16,111 +16,112 @@
  *
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
- * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  *
  */
 
-#include "getopt.h"
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
 #include "pi-source.h"
-#include "pi-socket.h"
 #include "pi-expense.h"
 #include "pi-dlp.h"
 #include "pi-header.h"
+#include "pi-userland.h"
 
-/* Declare prototypes */
-static void display_help(char *progname);
-void print_splash(char *progname);
-int pilot_connect(char *port);
-
-struct option options[] = {
-	{"port",        required_argument, NULL, 'p'},
-	{"help",        no_argument,       NULL, 'h'},
-	{"version",	no_argument,       NULL, 'v'},
-	{NULL,          0,                 NULL, 0}
-};
-
-static const char *optstring = "p:hv";
-
-static void display_help(char *progname)
+int main(int argc, const char *argv[])
 {
-	printf("   Export Palm Expense application database data into text format\n\n");
-	printf("   Usage: %s -p <port>\n\n", progname);
-	printf("   Options:\n");
-	printf("     -p, --port <port>       Use device file <port> to communicate with Palm\n");
-	printf("     -h, --help              Display help information for %s\n", progname);
-	printf("     -v, --version           Display %s version information\n\n", progname);
-	printf("   Examples: %s -p /dev/pilot\n\n", progname);
-
-	return;
-}
-
-int main(int argc, char *argv[])
-{
-	int 	c,		/* switch */
-		db,
+	int 	db,
 		i,
 		ret,
+		c		= -1,
 		sd 		= -1;
-	char 	*progname 	= argv[0],
-		*port 		= NULL;
+	enum { mode_none, mode_write = 257 } run_mode = mode_none;
+
 	char buffer[0xffff];
 	char buffer2[0xffff];
+	pi_buffer_t	*recbuf,
+		*appblock;
+
 	struct 	PilotUser User;
 	struct 	ExpenseAppInfo tai;
 	struct 	ExpensePref tp;
-		
-	while ((c = getopt_long(argc, argv, optstring, options, NULL)) != -1) {
-		switch (c) {
 
-		case 'h':
-			display_help(progname);
-			return 0;
-		case 'v':
-			print_splash(progname);
-			return 0;
-		case 'p':
-			port = optarg;
+	poptContext po;
+
+	struct poptOption options[] = {
+		USERLAND_RESERVED_OPTIONS
+		{"write",'w', POPT_ARG_NONE,NULL,mode_write,"Write data"},
+	        POPT_TABLEEND
+	};
+
+	po = poptGetContext("read-expenses", argc, argv, options, 0);
+	poptSetOtherOptionHelp(po,"\n\n"
+		"   Export Palm Expense application database data into text format\n\n");
+
+	if ((argc < 2)) {
+		poptPrintUsage(po,stderr,0);
+		return 1;
+	}
+
+	while ((c = poptGetNextOpt(po)) >= 0) {
+		switch(c) {
+		case mode_write :
+			if (run_mode == mode_none) {
+				run_mode = c;
+			} else {
+				if (c != run_mode) {
+					fprintf(stderr,"   ERROR: Specify exactly one of -w.\n");
+					return 1;
+				}
+			}
 			break;
 		default:
-			display_help(progname);
-			return 0;
+			fprintf(stderr,"   ERROR: Unhandled option %d.\n",c);
+			return 1;
 		}
 	}
 
-        sd = pilot_connect(port);
+	if (c < -1) {
+		plu_badoption(po,c);
+	}
+	if (mode_none == run_mode) {
+		fprintf(stderr,"   ERROR: Specify --write (-w) to output data.\n");
+		return 1;
+	}
+
+        sd = plu_connect();
         if (sd < 0)
                 goto error;
 
-        if (dlp_ReadUserInfo(sd, &User) < 0)
+        if (dlp_ReadUserInfo(sd, &User) < 0) {
                 goto error_close;
+	}
+
 
-	
 	/* Note that under PalmOS 1.x, you can only read preferences before
-	   the DB is opened 
+	   the DB is opened
 	 */
 	ret = dlp_ReadAppPreference(sd, Expense_Creator, Expense_Pref, 1,
 				  0xffff, buffer, 0, 0);
-		
+
 	/* Open the Expense database, store access handle in db */
 	if (dlp_OpenDB(sd, 0, 0x80 | 0x40, "ExpenseDB", &db) < 0) {
-		printf("Unable to open ExpenseDB");
+		fprintf(stderr,"   ERROR: Unable to open ExpenseDB on Palm.\n");
 		dlp_AddSyncLogEntry(sd, "Unable to open ExpenseDB.\n");
-		exit(1);
+		goto error_close;
 	}
-		
+
 	if (ret >= 0) {
 		unpack_ExpensePref(&tp, buffer, 0xffff);
 		i = pack_ExpensePref(&tp, buffer2, 0xffff);
-		
+
 #ifdef DEBUG
 		fprintf(stderr, "Orig prefs, %d bytes:\n", ret);
-		dumpdata(buffer, ret);
+		pi_dumpdata(buffer, ret);
 		fprintf(stderr, "New prefs, %d bytes:\n", i);
-		dumpdata(buffer2, i);
+		pi_dumpdata(buffer2, i);
 #endif
 		printf("Expense prefs, current category %d, default currency %d\n",
 			tp.currentCategory, tp.defaultCurrency);
@@ -135,15 +136,17 @@ int main(int argc, char *argv[])
 		}
 		printf("\n\n");
 	}
-		
-	ret = dlp_ReadAppBlock(sd, db, 0, buffer, 0xffff);
-	unpack_ExpenseAppInfo(&tai, buffer, 0xffff);
+
+	appblock = pi_buffer_new(0xffff);
+	ret = dlp_ReadAppBlock(sd, db, 0, 0xffff, appblock);
+	unpack_ExpenseAppInfo(&tai, appblock->data, appblock->used);
+	pi_buffer_free(appblock);
 #ifdef DEBUG
 	i = pack_ExpenseAppInfo(&tai, buffer2, 0xffff);
 	printf("Orig length %d, new length %d, orig data:\n", ret, i);
-	dumpdata(buffer, ret);
+	pi_dumpdata(buffer, ret);
 	printf("New data:\n");
-	dumpdata(buffer2, i);
+	pi_dumpdata(buffer2, i);
 #endif
 	printf("Expense app info, sort order %d\n", tai.sortOrder);
 	printf(" Currency 1, name '%s', symbol '%s', rate '%s'\n",
@@ -158,14 +161,16 @@ int main(int argc, char *argv[])
 	printf(" Currency 4, name '%s', symbol '%s', rate '%s'\n\n",
 		tai.currencies[3].name, tai.currencies[3].symbol,
 		tai.currencies[3].rate);
-		
+
+	recbuf = pi_buffer_new (0xffff);
+
 	for (i = 0;; i++) {
 		int 	attr,
 			category;
 		struct Expense t;
 
 		int len =
-		    dlp_ReadRecordByIndex(sd, db, i, buffer, 0, 0, &attr,
+		    dlp_ReadRecordByIndex(sd, db, i, recbuf, 0, &attr,
 					  &category);
 
 		if (len < 0)
@@ -176,17 +181,19 @@ int main(int argc, char *argv[])
 		    || (attr & dlpRecAttrArchived))
 			continue;
 
-		unpack_Expense(&t, buffer, len);
+		unpack_Expense(&t, recbuf->data, recbuf->used);
 		ret = pack_Expense(&t, buffer2, 0xffff);
 #ifdef DEBUG
 		fprintf(stderr, "Orig length %d, data:\n", len);
-		dumpdata(buffer, len);
+		pi_dumpdata(buffer, len);
 		fprintf(stderr, "New length %d, data:\n", ret);
-		dumpdata(buffer2, ret);
-#endif 
+		pi_dumpdata(buffer2, ret);
+#endif
 		printf("Category: %s\n", tai.category.name[category]);
-		printf("  Type: %3d\n  Payment: %3d\n  Currency: %3d\n",
-			t.type, t.payment, t.currency);
+		printf("  Type: %s (%3d)\n  Payment: %s (%3d)\n  Currency: %3d\n",
+			ExpenseTypeNames[t.type],t.type,
+			ExpensePaymentNames[t.payment],t.payment,
+			t.currency);
 		printf("  Amount: %s\n  Vendor: %s\n  City: %s\n",
 			t.amount ? t.amount : "<None>",
 			t.vendor ? t.vendor : "<None>",
@@ -198,8 +205,9 @@ int main(int argc, char *argv[])
 		printf("\n");
 
 		free_Expense(&t);
-	}		
-		
+	}
+	pi_buffer_free(recbuf);
+
 	/* Close the database */
 	dlp_CloseDB(sd, db);
 
@@ -210,7 +218,7 @@ int main(int argc, char *argv[])
 
 error_close:
         pi_close(sd);
-        
+
 error:
         return -1;
 }
